<?php
/*
Plugin Name: Upload Plus
Plugin URI: none
Description: This is a enhanced version of the upload feature
Author: Bill Bellon
Version: 0.7
Author URI: http://www.ssec.wisc.edu/~billb/
*/ 

/*---------------------------------------------------------------------------
TO DO LIST:
- add a single function that cleans ALL input
- decide whether to try to move file before or after database update and put in logic
  to recover from a failure - do an if statement that tests SQL statement AND 
  tests to see if file can be moved - if both are true, then update database and move
  file
- make "checkin" in filelist only show up if you are the person who checked it out - otherwise,
  just put some text saying "checked out" in that column for that file if checked out by someone
  else
- make sure to use is_null() instead of isset() when appropriate/needed and vice versa
- make sure I'm using compound boolean statements correctly
- CLEAN UP / ORGANIZE / SIMPLIFY CODE AND TEST

- REFER TO NOTEBOOK AND PDA FOR ADDITIONAL TO DO ITEMS
- make sure to explicitly stop uploading of critical system files - file.php, index.*,
  .ht*, ...
- use a real creationDate
- make sure that id of new uploaded file is not larger than 5 digits
- put navigation buttons/links back to file list on results pages
- refer to $HOME/notebook/notes.txt 2005-11-18 notes for tests to finish on completed 
  plugin that test error checking/catching with the upload functionality

SAVE FOR FUTURE VERSIONS:
- check to see if file already exists (in database) and act appropriately 
  (do this after integrating into uploadplus) - backup file to something like
   fileid-filename.ext.version2
   and rotate versions so that there are no more than X number of versions
   THEN, allow new uploaded file to overwrite existing one
- change file.php so that it automatically downloads file to computer with "fileNameInDB" filename
- add server response header so that when you click on view, you don't get cached file

DONE:
- USE $_FILES[$up_filevar]['filename'] IN PLACE OF $_POST['up_filename'] WHEN UPDATING
  DATABASE - AND REMOVE THE up_filename FIELD 
- for a file that is checked out allow only person who has checked out a file be able to check it in 
  AND to edit it's properties - for files that are checked in, allow anyone to edit properties
- add checkin capability
- see if I can make a popup window/page that has none of the admin-header and footer
  code BUT has access to the variables/functions used to determine if a user is logged
  in or not ==> ask on forum; otherwise, continue to integrate upload form into this page.
  CHANGED MY MIND - do the uploading in same window
- might want to always rename uploaded file "filenameB" to 
  fileid-filenameA.ext (and fileid-filenameA.ext.versionX) if file already exists
  NOT GOING TO DO THIS
- add file id to filename
- lock edit properties if file is checked out
- modify file.php and the rest of the code so that it knows to add 'file'.$id.'.'. to
  beginning of filename stored in database
- show only the file that was uploaded/checked in in up_receiveUpload() rather than the entire
  file list => NOT APPLICABLE

---------------------------------------------------------------------------*/

$up_filevar = 'up_userfile';

//====================================================================================
// FUNCTIONS: BEGIN
//====================================================================================
include_once('functions.inc');

function uploadplus_add_admin_menu_pages() {
  // not sure why I need to specify "__FILE__" for the file parameter, but that is what all of the examples
  // use
  add_management_page('UploadPlus', 'UploadPlus', 8, __FILE__, 'up_manager');
}

/* -----------------------------------------------------
------------------------------------------------------- */
function up_manager() {
  global $user_level;

  if ($user_level < 6) {
    up_authorizationFailure();
  }

  if (isset($_GET['up_action'])) {
    $up_action=up_clean($_GET['up_action'], 20);

    if ($up_action == 'upload') { // a file was uploaded
      if ( isset($_POST['up_id']) ) {
        $up_id=up_clean($_POST['up_id'], 6);
        up_checkId($up_id);
        up_receiveUpload($up_id);
      } else {
        up_receiveUpload();
      }
    } else if ($up_action == 'showUploadForm') { // show form for uploading a new file
      up_showUploadForm();
    } else {  // file was not uploaded
      if ( !isset($_GET['up_id']) ) {
        die('No id');
      }
      if (isset($_GET['up_filename'])) {
        $up_filename = up_clean($_GET['up_filename'], 255);
      }
      $up_id=up_clean($_GET['up_id'], 6);
      up_checkId($up_id);
	
      switch ($up_action) {
      case "delete":
        up_deleteFile($up_id);
        break;
      case "edit":
        up_editProperties($up_id);
        break;
      case "checkin":
        up_showUploadForm($up_id);
        break;
      case "checkout":
        up_checkOut($up_id, $up_filename);
        break;
      default:
        die("Invalid action");
        break;
      }
    }
  } else {
    up_showFileList();
  }

}

// -----------------------------------------------------
function up_receiveUpload($id=NULL) {
  global $up_filevar;

  echo '<div class="wrap"><pre>'."\n";
  $errormessage = array(
       0=>"There is no error, the file uploaded with success",
       1=>"The uploaded file exceeds the upload_max_filesize directive in php.ini",
       2=>"The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form",
       3=>"The uploaded file was only partially uploaded",
       4=>"No file was uploaded",
       6=>"Missing a temporary folder");

  $uploaddir = '/var/apache/test/htdocs/wp-test/wp-content/files/';

  if ( isset($_FILES[$up_filevar]['error']) && $_FILES[$up_filevar]['error'] == UPLOAD_ERR_OK ) {
    // remove invalid characters from filename and clean up
    $_FILES[$up_filevar]['name'] = basename_($_FILES[$up_filevar]['name']);                  // strip off path
    $_FILES[$up_filevar]['name'] = trim($_FILES[$up_filevar]['name']);                                       // remove leading and ending white-space
    $_FILES[$up_filevar]['name'] = preg_replace('/[^a-zA-Z\-_\.0-9]/', '_', $_FILES[$up_filevar]['name']);  // replace UNIX hostile characters
    $_FILES[$up_filevar]['name'] = preg_replace('/_{1,}/', '_', $_FILES[$up_filevar]['name']);              // concatenate multiple "_" into one "_"
    $_FILES[$up_filevar]['name'] = up_fileNameInDB($_FILES[$up_filevar]['name']);  // remove file#####
  
    // check to see if file is valid
    $maxfilesize = 8*1048576; // 8 MB
    $allowedExtensions = array('jpeg', 'jpg', 'gif', 'png', 'doc', 'pdf', 'txt', 'ppt', 'xls', 'zip');
    $file_is_ok = up_checkfile($_FILES[$up_filevar]['name'], $allowedExtensions, $maxfilesize);

    if ( $file_is_ok[0] != 'yes' ) { // ERROR: there was a problem with file
      echo 'ERROR: The uploaded file was not valid (file_is_ok='.$file_is_ok[0].'); Reasons: ';
      for ($ii=1; $ii < count($file_is_ok); $ii++) {
        echo $file_is_ok[$ii] . ' : ';
      }
    } else { // file is OK
      if (is_null($id)) {
        $returnID = up_updateDatabase();
        up_checkId($returnID);      
      } else {
        $connection = up_getConnection();
        $returnID = up_updateDatabase($connection, $id);
        if ($returnID != $id) {
          die("ERROR: return ID and requested ID do not match for uploaded file");
        }
        mysql_close($connection);
      }

      $uploadfile = $uploaddir . 'file' . sprintf("%05d", $returnID) . '.' . $_FILES[$up_filevar]['name'];  // REMOVE THIS AFTER DEBUGGING
//      $uploadfile = up_realFileName($_FILES[$up_filevar]['name'], $returnID);  // ADD THIS IN AFTER DONE DEBUGGING
      if ( is_uploaded_file($_FILES[$up_filevar]['tmp_name']) && move_uploaded_file($_FILES[$up_filevar]['tmp_name'], $uploadfile) ) {
         echo "File is valid, and was successfully uploaded.\n";
      } else {
         echo "ERROR: There was an error and the file was not uploaded!\n";
      }
    }
  } else {
    echo "ERROR: file was not uploaded.  Reason: ".$errormessage[$_FILES[$up_filevar]['error']]."\n";
  }

  echo "</pre></div>\n";

  up_showFileList();
}

//----------------------------------------------------------------------------------
function basename_($filen) {
  // remove everything at the beginning of the string that isn't an alpha-numeric character
  $filen = preg_replace('/^[^a-zA-Z0-9]*/', '', $filen);
  
  if ( !(preg_match('/^[a-zA-Z0-9]/', $filen)) ) {
    die("ERROR: uploaded filename had fatal flaws\n");
  }
 
  return $filen;

}

//------------------------------------------------------------------
function up_checkfile($filename, $allowedExtensions, $maxfilesize=NULL) {
  global $up_filevar;
  $ok[0] = 'null';

  // CHECK FILE TYPE AND EXTENSION
  $array = explode(".", $filename);
  $nr    = count($array);
  $ext  = $array[$nr-1];
  if ( in_array($ext, $allowedExtensions) ) {
    $ok[0] = 'yes';
  } else {
    $ok[0] = 'no';
    $ok[] = 'uploaded file type is not allowed';
  }

  // CHECK FILE SIZE
  if ( $_FILES[$up_filevar]['size'] == 0 ) {  // make sure filesize is non-zero
      $ok[0] = 'no';
      $ok[] = 'uploaded file size is zero';
  }

  if(isset($maxfilesize)) {
    if($_FILES[$up_filevar]["size"] > $maxfilesize) {  // make sure file is not too large
      $ok[0] = 'no';
      $ok[] = 'uploaded file is too large';
    }
  }

  return $ok;
}
/* -----------------------------------------------------
------------------------------------------------------- */
function up_showUploadForm($id=NULL) {
  global $up_filevar;

  echo '<div class="wrap">'."\n";
  $title='';
  $description='';
  $label = 'Select a File to Upload';
  $checkinFileID = '';
  $oldFileName = '';

  if (isset($id) && is_numeric($id)) {
    $connection = up_getConnection();
    $username = up_getUserLogin();

    // READ DATA FROM DATABASE AND DISPLAY IN FORM
    $sql = 'SELECT title, description, filename FROM wp_uploadplus WHERE id='.$id." AND checkedout='".$username."'";
    if (!($result = @mysql_query($sql, $connection))) {
      up_showerror();
    }

    $num_rows = @mysql_num_rows($result);
    if ($num_rows == 1) {
      $row = mysql_fetch_array($result, MYSQL_ASSOC);
      $title = $row['title'];
      $description = $row['description'];
      $label = 'Select File to Checkin';
      $checkinFileID = '<input type="hidden" name="up_id" id="up_id" value="'.$id.'">'."\n";
      $oldFileName = '<input type="hidden" name="up_oldfilename" id="up_oldfilename" value="'.$row['filename'].'">'."\n";
    } else if ($num_rows > 1) {
       up_showerror('ERROR: Too many rows retrieved for requested ID');
    } else {
       up_showerror('ERROR: Requested file to checkin is not in database - most likely cause: file is checked out by someone else');
    }
	
    mysql_free_result($result); 
    mysql_close($connection);
  }
?>
  <div class="up_nav"><a href="<?php print(UP_URL); ?>">&lt;&lt; Back to Manage Files</a></div>
  <h2>Upload Plus</h2>
  <h3 style="margin-bottom: 0px; padding-bottom: 0px;">Upload a file</h3>
  <form enctype="multipart/form-data" action="<?php echo UP_URL; ?>&amp;up_action=upload" method="POST" style="margin-top: 0px; padding-top: 0px;">
  <table class="uploadtable"><thead>
    <tr><th>Title</th><th>Description</th></tr></thead><tbody><tr>
      <td><input type="text" name="up_title" id="up_title" maxlength="255" size="25" value="<?php echo $title;?>"></td>
      <td><input type="text" name="up_description" id="up_description" maxlength="255" size="25" value="<?php echo $description;?>"></td>
    </tr></tbody></table>
    <input type="hidden" name="MAX_FILE_SIZE" value="8000000" />
    <?php echo $label;?>: <input name="<?php echo $up_filevar;?>" type="file" />
    <?php echo $checkinFileID; ?>
    <?php echo $oldFileName; ?>
    <input type="submit" value="Upload File" name="submit" id="submit" />
  </form>
  </div>
<?php
}

/* -----------------------------------------------------
Show the the list of uploaded files
------------------------------------------------------- */
function up_showFileList() {
  $connection = up_getConnection();

?>
  <div class="wrap">
  <h2>Upload Plus</h2>
  <a href="<?php print(UP_URL.'&amp;up_action=showUploadForm'); ?>"><input type="button" value="Upload a new file"></a>

  <h3>Manage files</h3>
<?php
  // GET RID OF THIS 1st SQL CONNECTION AND MODIFY SQL CONNECTION BELOW TO INCLUDE 
  // LOGIC FOR DECIDING WHETHER OR NOT THERE ARE ANY RESULTS
  $sql = 'SELECT COUNT(id) FROM wp_uploadplus';
  if (!($result = @mysql_query($sql, $connection))) {
    up_showerror();
  }
  if (mysql_result($result, 0, 0) < 1):
    print('No files yet.');
  else:
?>
    <table class="uploadtable">
    <thead>
      <tr><th>ID</th><th>Title</th><th>File Name</th><th>Description</th><th>Size (bytes)</th><th>Created</th><th>Modified</th>
          <th><!--View/Download--></th><th><!--Checkout and Edit / Checkin--></th><th><!--Edit Properties--></th>
          <th><!--Delete--></th></tr>
    </thead>
    <tbody>
<?php
    $sql = 'SELECT id, title, filename, description, created, modified, checkedout FROM wp_uploadplus ORDER BY id DESC';
	
    if (!($result = @mysql_query($sql, $connection))) {
      up_showerror();
    }
	
    $rowCount = 0;
    while ($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
      $isEven = ($rowCount % 2) == 0;
      $rowClass = $isEven ? " class=\"alternate\"" : "";

      print('<tr'.$rowClass.'>');
      print('<td>'.$row['id'].'</td>');
      print("<td>".$row["title"]."</td>");
      print("<td>".$row["filename"]."</td>");
      print("<td>".$row["description"]."</td>");
      print('<td>' . up_getFileSize($row['filename']) . '</td>');
      print("<td>".$row["created"]."</td>");
      print("<td>".$row["modified"]."</td>");
//      print('<td><a href="/wp-test/wp-content/files/'.$row['filename'].'">View</a></td>');
      print('<td><a href="/wp-test/wp-content/files/file.php?up_id='.$row['id'].'" target="_blank">View</a></td>');
      if ($row["checkedout"] != '') { // checked out
        print('<td><a href="'.UP_URL.'&amp;up_action=checkin&amp;up_id='.$row['id'].'" title="checked out by: '.$row['checkedout'].'">Checkin</a></td>');
        print('<td><span title="disabled - file is checked out" style="color: #bbbbbb;">Edit Properties</span></td>');
        print('<td><span title="disabled - file is checked out" style="color: #bbbbbb;">Delete</span></td>');
      } else { // checked in
        print('<td><a href="' . UP_URL . '&amp;up_action=checkout&amp;up_id='.$row['id'].
              '&amp;up_filename='.$row['filename'].'">Checkout &amp; edit</a></td>');
        print('<td><a href="'.UP_URL.'&amp;up_action=edit&amp;up_id='.$row['id'].'">Edit Properties</a></td>');
        print('<td><a href="'.UP_URL.'&amp;up_action=delete&amp;up_id='.$row['id'].'">Delete</a></td>');
      }

      print("</tr>\n");

      $rowCount++;
    }

    mysql_free_result($result);
    mysql_close($connection);
?>
    </tbody>
    </table>
<?php
  endif;
?>
  </div>
<?php
}
/* -----------------------------------------------------
------------------------------------------------------- */
function up_deleteFile($id) {
  echo '<div class="wrap">'."\n";
  echo '<div class="up_nav"><a href="'.UP_URL.'">&lt;&lt; Back to Manage Files</a></div>'."\n";
  echo "<h2>Upload Plus: Edit Properties</h2>\n";
  $connection = up_getConnection();

  $sql = 'SELECT filename FROM wp_uploadplus WHERE id='.$id." AND checkedout=''";
  if (!($result = @mysql_query($sql, $connection))) {
    up_showerror();
  }
  if (mysql_num_rows($result) == 1) {
    $filename = mysql_result($result, 0, 0);
    if (up_file_exists($filename, $id)) {
      // DELETE FROM DATABASE
      $sql = 'DELETE FROM wp_uploadplus WHERE id='.$id;
      @mysql_query($sql, $connection); 
      if (@mysql_affected_rows() != 1) {
        //TO DO: rename the file back to old filename
        die("ERROR: could not delete record from database\n");
      }

      // DELETE ACTUAL FILE
      if (!up_removeFile($filename, $id)) {
        die("could not delete file");
      }
    } else {
      die("requested file does not exist");
    }
  } else {
    die("requested file to delete does not exist in database or is checked out");
  }
  echo '<p>The file: '.$filename.' ('.up_realFileName($filename, $id).' ; id='.$id.') has been deleted and removed from the database</p>'."\n";

  mysql_free_result($result); 
  mysql_close($connection);
  echo "</div>\n";
}

/* -----------------------------------------------------
------------------------------------------------------- */
function up_editProperties($id) {

  echo '<div class="wrap">'. "\n";
  echo '<div class="up_nav"><a href="'.UP_URL.'">&lt;&lt; Back to Manage Files</a></div>'."\n";

  echo "<h2>Upload Plus: Edit Properties</h2>\n";
  $connection = up_getConnection();

  if (isset($POST['up_id']) && $id != $_POST['up_id']) {
    die("id's do not match\n");
  }

  // IF DATA HAS BEEN SUBMITTED, WRITE TO DATABASE FIRST
  if ($_POST['up_editSubmitted']) {
    up_updateDatabase($connection, $id);
  }

  // READ DATA FROM DATABASE AND DISPLAY IN FORM
  $sql = 'SELECT title, filename, description, created, modified, checkedout FROM wp_uploadplus WHERE id='.$id." AND checkedout=''";
  if (!($result = @mysql_query($sql, $connection))) {
    up_showerror();
  }

  $num_rows = @mysql_num_rows($result);
  if ($num_rows == 1) {
    $row = mysql_fetch_array($result, MYSQL_ASSOC);
    print('<form action="'.UP_URL.'&amp;up_action=edit&amp;up_id='.$id.'" method="post">'."\n");
    print ('<table><thead>
      <tr><th>ID</th><th>Title</th><th>File Name</th><th>Description</th><th>Size (bytes)</th><th>Created</th><th>Modified</th>
          <th><!--Checkout and Edit / Checkin--></th></tr></thead><tbody>');
    print("<tr>\n");
    print('<td>'.$id.'<input type="hidden" name="up_id" id="up_id" value="'.$row['id'].'"></td>'."\n");
    print('<td><input type="text" name="up_title" id="up_title" value="'.$row['title'].'" maxlength="255" size="25"></td>'."\n");
    print('<td><input type="text" name="up_filename" id="up_filename" value="'.$row['filename'].'" maxlength="255" size="25"></td>'."\n");
    print('<td><input type="text" name="up_description" id="up_description" value="'.$row['description'].'" maxlength="255" size="25"></td>'."\n");
    print('<td>' . up_getFileSize($row['filename']) . '</td>'."\n");
    print("<td>".$row["created"]."</td>\n");
    print("<td>".$row["modified"]."</td>\n");
    if ($row["checkedout"] != '') { // checked out
      print('<td>Checked out by '.$row['checkedout']."</td>\n");
    } else {  // checked in
      print("<td>&nbsp;</td>\n");
    }
    print("</tr></tbody></table>\n");
    print('<div style="text-align: center; margin-top: 10px;"><input type="submit" name="up_submit_edit" value="Submit changes"></div>'."\n");
    print('<input type="hidden" name="up_editSubmitted" id="up_editSubmitted" value="true">'."\n");
    print('<input type="hidden" name="up_oldfilename" id="up_oldfilename" value="'.$row['filename'].'">'."\n");
    print("</form>\n");
  } else if ($num_rows > 1) {
     up_showerror('ERROR: Too many rows retrieved for requested ID');
  } else {
     up_showerror('ERROR: no data returned for requested ID, possibly because file is checked out by someone else or because ...');
  }
	
  mysql_free_result($result); 
  mysql_close($connection);

  echo "</div>\n";

}

/* -----------------------------------------------------
------------------------------------------------------- */
function up_updateDatabase($connection=NULL, $id=NULL) {
  global $up_filevar;
  $deletedFileName = '';

  $needToCloseConnection = false;
  if (is_null($connection)) { // NO database connection inputted into function
    $connection = up_getConnection();
    $needToCloseConnection = true;
  }

  $_POST['up_title'] = up_clean($_POST['up_title'], 255);
  $_POST['up_description'] = up_clean($_POST['up_description'], 255);

  $creationDate='2999-12-12 12:12:12';
  if ( is_null($id) ) { // NO ID
    $sql = "INSERT INTO wp_uploadplus VALUES(NULL, '".$_POST['up_title']."', '".$_FILES[$up_filevar]['name'].
           "', '".$_POST['up_description']."', '".$creationDate."', NULL, '', NULL, NULL)";
  } else { // ID was supplied
    if ($_GET['up_action'] == 'upload') {
      // REMINDER: change up_userfile to up_filename here and elsewhere so that everything is consistent and so 
      // that I can simplify this code
      $_FILES[$up_filevar]['name'] = up_clean($_FILES[$up_filevar]['name'], 255);
      $_FILES[$up_filevar]['name'] = addslashes($_FILES[$up_filevar]['name']);
      $_FILES[$up_filevar]['name'] = preg_replace('/^[^a-zA-Z0-9]*/', '', $_FILES[$up_filevar]['name']);
      $sql = "UPDATE wp_uploadplus SET title='".$_POST['up_title']."', filename='".$_FILES[$up_filevar]['name']."', description='".
             $_POST['up_description']."', modified=NULL, checkedout='' WHERE id=".$id;

      // MOVE OLD FILENAME FOR DELETION IF CHECKED IN FILENAME IS DIFFERENT
      if ($_POST['up_oldfilename'] != $_FILES[$up_filevar]['name']) {
        $deletedFileName = $_POST['up_oldfilename'] . '.DELETED';
        up_renameFile($_POST['up_oldfilename'], $deletedFileName, $id);
      }
    } else {
      $_POST['up_filename'] = up_clean($_POST['up_filename'], 255);
      $_POST['up_filename'] = addslashes($_POST['up_filename']);
      $_POST['up_filename'] = preg_replace('/^[^a-zA-Z0-9]*/', '', $_POST['up_filename']);
      $sql = "UPDATE wp_uploadplus SET title='".$_POST['up_title']."', filename='".$_POST['up_filename']."', description='".
             $_POST['up_description']."' WHERE id=".$id." AND checkedout=''";
      // TRY TO RENAME/MOVE FILE IF FILENAME HAS CHANGED
      if ($_POST['up_oldfilename'] != $_POST['up_filename']) {
        up_renameFile($_POST['up_oldfilename'], $_POST['up_filename'], $id);
      }
    }
  }

  @mysql_query($sql, $connection); 
  if (@mysql_affected_rows() != 1) {
    //rename the file back to old filename
    die("ERROR: properties not updated\n");
  }

  // DELETE OLD FILENAME IF CHECKED IN FILENAME IS DIFFERENT
  if ($deletedFileName != '') {
    if (up_file_exists($deletedFileName, $id) && up_removeFile($deletedFileName, $id)) { 
//    if (up_file_exists($deletedFileName, $id) && false) {   // TEST THIS AGAIN
      // come up with a better logic for this
    } else {
      die("ERROR: could not delete old version of file"); 
    } 
  }

  if ($newid = mysql_insert_id($connection)) {
    $fileid = $newid;
  } else {
    $fileid = $id;
  }

  if ($needToCloseConnection) {
    mysql_close($connection);
  }

  return $fileid;
}

/* -----------------------------------------------------
------------------------------------------------------- */
function up_checkOut($id, $filename) {
  global $userdata;
  get_currentuserinfo();

  echo '<div class="wrap">'. "\n";
  echo '<div class="up_nav"><a href="'.UP_URL.'">&lt;&lt; Back to Manage Files</a></div>'."\n";
  $connection = up_getConnection();

//   echo '<div style="font-size: 90%; margin-bottom: 10px;"><a href="'.UP_URL.'">Back to file list</a></div>'."\n";
  // TRY TO CHECK OUT FILE AND DISPLAY RESULTS
  $sql = "UPDATE wp_uploadplus SET checkedout='".$userdata->user_login."' WHERE id=" . $id . " AND checkedout=''";
  if ( @mysql_query($sql, $connection) ):
    if (@mysql_affected_rows() == 1) {
      echo "<h2>Upload Plus: File check out</h2>\n";
      echo '<p>This file has been checked out.  Download file, make your changes, then check the file back in
               by clicking on the link entitled &quot;checkin&quot; on the File List page</p>'."\n";
//      echo '<p><a href="/wp-test/wp-content/files/' . $filename . '">Download file</a></p>' . "\n";
      echo '<p><a href="/wp-test/wp-content/files/file.php?up_id='.$id.'" target="_blank">Download file</a></p>'."\n";
    } else {
      echo '<h2>Upload Plus: File is <span style="color: red;">already checked out</span>!</h2>'."\n";
      echo '<p>This file is already checked out by: &nbsp;&nbsp;<span style="color: red; font-size: 115%;">'.$userdata->user_login."</span>.&nbsp;&nbsp;  Wait until this person has checked
               the file back in or contact the administrator if you encounter any problems.</p>\n";
    }
  else:
    up_showerror();
  endif;

//  mysql_free_result($result); GIVES ME AN ERROR MESSAGE ABOUT $result not being a valid resource
  mysql_close($connection);

  echo "</div>\n";
}

/* -----------------------------------------------------
------------------------------------------------------- */
function up_makefilelist($content = '') {
  $tagRegExp = '/<\!\-\-[ ]*UPLOADPLUS[ ]+ACTION="([a-zA-Z0-9 ]+)"[ ]+NUM="([a-zA-Z0-9 ]+)"[ ]*-->/i';
  if( preg_match($tagRegExp, $content, $matches) ) {
    $action=$matches[1];
    $num=$matches[2];

    if (strtolower($action) == 'filelist' && ($num >= 1 || strtolower($num) == 'all')) {
      $connection = up_getConnection();

      $sql = 'SELECT id, title, description FROM wp_uploadplus ORDER BY id DESC';

      if ($num == 'all') $num = 9999; // a really, really bad way to program, but i'm lazy
	
      if ($result = @mysql_query($sql, $connection)) {
        $rowCount = 1;
        $filelist = "<ul>\n";
        while ($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
          $filelist = $filelist . '<li><a href="/wp-test/wp-content/files/file.php?up_id='.$row['id'].'" title="'.$row['description'].'">'.$row['title']."</a></li>\n";
          $rowCount++;
          if ($rowCount > $num) break;
        }

        $content=preg_replace($tagRegExp, $filelist, $content);    

      }
      @mysql_free_result($result);
      @mysql_close($connection);
    }
  }

  return $content;
}

/* -----------------------------------------------------
------------------------------------------------------- */
function up_showPostForm() {
  /*
  DISABLED - NOT USED
  echo '<div class="wrap">
  <h2>UploadPlus: Upload a file</h2>
  PUT FORM FOR UPLOADING A FILE HERE AND MODIFY edit-form-advanced.php SO THAT A LINK TO
  FILE IS PUT IN POST (OR MAYBE, ADD A FIELD TO DATABASE FOR POSTS THAT LISTS ALL OF THE FILE
  ID\'S THAT ARE ATTACHED TO A POST AND PUT IN A FILTER THAT WILL GENERATE A LIST OF ATTACHED
  FILES FROM THIS FIELD.<br><br>ALSO PUT A BUTTON FOR LINKING TO A FILE THAT ALREADY IS UPLOADED
  IN THE POST.
  </div>'."\n";
  */
}

/* -----------------------------------------------------
------------------------------------------------------- */
function up_makeAttachFileMenu() {
  $connection = up_getConnection();
  $sql = 'SELECT id, filename, title, description FROM wp_uploadplus ORDER BY id DESC';

  echo '<script type="text/javascript">'."\n";
  if ($result = @mysql_query($sql, $connection)) {
    while ($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
      echo "edLinks[edLinks.length] = new edLink('".$row['title']."', '/wp-test/wp-content/files/file.php?up_id=".$row['id'].
            "', 0, ".$row['id'].", '".$row['filename']."');\n";
    }
  } 
  echo '</script>'."\n";
}

/* -----------------------------------------------------
------------------------------------------------------- */
function up_getUserLogin() {
  global $user_ID; 
  $user_data = get_userdata($user_ID);
  return $user_data->user_login;
}

//====================================================================================
// END OF FUNCTIONS
//====================================================================================

add_filter('the_content', 'up_makefilelist');
add_action('admin_menu', 'uploadplus_add_admin_menu_pages');

?>
